import React from 'react';
import { getStatus, createGame, gameInfo, joinGame, getGuesses, sendGuess } from "./services/ggsh.service";
import './App.css';

class InfoComponent extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      version: null,
      status: null,
    }
  }

  async componentDidMount() {
    const { version, status } = await getStatus();
    this.setState({loading: false, version, status});
  }

  render() {
    if (this.state.loading) {
      return <p>Connecting to back-end...</p>
    }

    if (this.state.status !== "healthy") {
      return <p>Unable to communicate with API server! Error: {this.state.status}</p>
    }
    
    return <p>API Version: {this.state.version}</p>
  }
}

const GameInfo = ({game}) => {
  const {
    gameId,
    creator,
    timer,
    coords,
    players
  } = game;

  return (
    <div className="GameInfo">
      <p>Game ID: {gameId}</p>
      <p>Creator: {creator}</p>
      <p>Time Limit: {timer}</p>
      <p>Coordinates: {JSON.stringify(coords)}</p>
      <p>Players: {JSON.stringify(players)}</p>
    </div>
  );
}

class GuessComp extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: true,
      guesses: null,
    }
  }

  async componentDidMount() {
    const { gameId, name } = this.props;
    const guesses = await getGuesses(gameId, name);
    this.setState({loading: false, guesses});
  }

  render() {
    const { loading, guesses } = this.state;

    if (loading) {
      return <p>Loading...</p>
    }

    const { name } = this.props;
    return <p>Guesses for {name}: {JSON.stringify(guesses)}</p>
  }
}

class GameMakerComp extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      loading: false,
      game: null,
    }
  }

  async handleGameCreate() {
    this.setState({loading: true});
    const gameId = await createGame("testing", 300);
    const game = await gameInfo(gameId);
    this.setState({loading: false, game});
  }

  async handleGameJoin() {
    this.setState({loading: true});
    const { gameId } = this.state.game
    await joinGame(gameId, "testing2");
    const game = await gameInfo(gameId);
    this.setState({loading: false, game});
  }

  async handleSendGuess() {
    this.setState({loading: true});
    const { gameId, coords } = this.state.game
    const { currentRound } = await getGuesses(gameId, "testing");
    const { lat, lng } = coords[currentRound];
    await sendGuess(gameId, "testing", currentRound, lat, lng);
    const game = await gameInfo(gameId);
    this.setState({loading: false, game});
  }

  render() {
    const { loading, game } = this.state;

    if (loading) {
      return <div><p>Loading...</p></div>
    }

    if (game) {
      return (
        <div>
          <GameInfo game={game}/>
          <p>Your Guesses:</p>
          <GuessComp gameId={game.gameId} name="testing"/>
          <button onClick={() => this.handleGameJoin()}>Add second user!</button>
          <button onClick={() => this.handleSendGuess()}>Send perfect guess!</button>
        </div>
      );
    }
    return <div><button onClick={() => this.handleGameCreate()}>Create Game!</button></div>
  }
}

const App = () => {
  return (
    <div className="App">
      <p>TerrAssumptions!</p>
      <hr/>
      <GameMakerComp/>
      <hr/>
      <InfoComponent/>
    </div>
  );
}

export default App;
